/*
 * =====================================================================================
 *
 *       Filename:  main.cpp
 *
 *    Description:  Simple solve for strumpack.
 *
 *        Version:  1.0
 *        Created:  14.12.2019 18,28,11
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  YOUR NAME (), 
 *   Organization:  IICT
 *
 * =====================================================================================
 */
#include <cmath>
#include <iostream>
using namespace std;

#include "dense/DistributedMatrix.hpp"
#include "HSS/HSSMatrixMPI.hpp"
using namespace strumpack;
using namespace strumpack::HSS;

int main(int argc, char **argv)
{
    MPI_Init(&argc, &argv);
    if (argc != 2) {
        if (!mpi_rank()) {
            printf("usage:\n"
                   "    main <number-of-unknowns>\n\n"
                   "    IMPORTANT: files D_<number-of-unknowns>.txt and sv_<number-of-unknowns>.txt must be present in results\n");
        }
        exit(EXIT_FAILURE);
    }

    int size = atoi(argv[1]);
    HSSOptions<double> hss_opts;
    hss_opts.set_verbose(false);

    BLACSGrid grid(MPI_COMM_WORLD);
    DistributedMatrix<double> A;
    A = DistributedMatrix<double>(&grid, size, size);

    DistributedMatrix<double> sv(&grid, size, 1);

    char filename[500];
    (void) sprintf(filename, "./results/D_%d.txt", size);
    {
        FILE *file = fopen(filename, "r");
        if (file == NULL) {
            fprintf(stderr, "%s:%d Cannot read file %s!\n",
                    __FILE__, __LINE__, filename);
            MPI_Abort(MPI_COMM_WORLD, 1);
        }
        double value;
        for (int i=0; i<size; i++) {
            for (int j=0; j<size; j++) {
                fscanf(file, "%lf", &value);
                A.global(i, j, value);
            }
        }
    }
    {
        (void) sprintf(filename, "./results/sv_%d.txt", size);
        FILE *file = fopen(filename, "r");
        if (file == NULL) {
            fprintf(stderr, "%s:%d Cannot read file %s!\n",
                    __FILE__, __LINE__, filename);
            MPI_Abort(MPI_COMM_WORLD, 1);
        }
        double value;
        for (int i=0; i<size; i++) {
            fscanf(file, "%lf", &value);
            sv.global(i, 0, value);
        }
    }

//    if (!mpi_rank()) {
        printf("%d %d values:\n"
               "   %lf %lf %lf \n"
               "   %lf %lf %lf \n"
               "   %lf %lf %lf \n",
               grid.prow(),
               grid.pcol(),
               A(0,0),
               A(0,1),
               A(0,1),
               A(1,0),
               A(1,1),
               A(1,2),
               A(2,0),
               A(2,1),
               A(2,2)
              );
        if (grid.pcol() == 0) {
            printf("%d %d sv alues:\n"
                   "   %lf \n"
                   "   %lf \n"
                   "   %lf \n",
                   grid.prow(),
                   grid.pcol(),
                   sv(0,0),
                   sv(1,0),
                   sv(2,0)
                  );
        }
//    }
    //

    scalapack::Cblacs_exit(1);
    MPI_Finalize();
    return 1;
}
